/*
 * Copyright (C) 2004-2012 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include "ZLDialog.h"

ZLDialog::ZLDialog() {
}

ZLDialog::~ZLDialog() {
	delete myTab;
}

void ZLDialog::addOption(const std::string &name, const std::string &tooltip, ZLOptionEntry *entry) {
	myTab->addOption(name, tooltip, entry);
}

void ZLDialog::addOption(const ZLResourceKey &key, ZLOptionEntry *entry) {
	myTab->addOption(key, entry);
}

void ZLDialog::addOption(const ZLResourceKey &key, ZLSimpleOption &option) {
	myTab->addOption(key, option);
}

void ZLDialog::acceptValues() {
	myTab->accept();
}

const ZLResource &ZLDialog::resource(const ZLResourceKey &key) const {
	return myTab->resource(key);
}
