/*
 * Copyright (C) 2004-2012 Geometer Plus <contact@geometerplus.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef __AUTHORINFODIALOG_H__
#define __AUTHORINFODIALOG_H__

#include <string>

#include "../library/Lists.h"

class ZLDialog;
class AuthorNameEntry;
class AuthorSortKeyEntry;

class AuthorInfoDialog {

public:
	static bool run(shared_ptr<Author> author);

private:
	AuthorInfoDialog(shared_ptr<Author> author);

	ZLDialog &dialog();

public:
	shared_ptr<Author> initialAuthor();
	const AuthorList &authors() const;

	std::string &name();
	std::string &sortKey();

	AuthorNameEntry &nameEntry();
	AuthorSortKeyEntry &sortKeyEntry();

private:
	const shared_ptr<Author> myInitialAuthor;
	AuthorList myAuthors;

	shared_ptr<ZLDialog> myDialog;

	AuthorNameEntry *myNameEntry;
	AuthorSortKeyEntry *mySortKeyEntry;

	std::string myName;
	std::string mySortKey;
};

#endif /* __AUTHORINFODIALOG_H__ */
